import java.net.*;
import java.util.*;
import java.io.*;

public class Cliente implements Runnable {
   private Socket servidor;
   private DataInputStream in;
   private DataOutputStream out;

   public Cliente(String endereco, int porta) throws Exception {
      // conectando-se e criando IO 
      this.servidor = new Socket(endereco, porta);
      this.in = new DataInputStream( this.servidor.getInputStream());
      this.out = new DataOutputStream( this.servidor.getOutputStream());

      
      String entrada = in.readLine(); // recebe valor e passo 
      Mensagem p = Mensagem.stringToMensagem(entrada);
   
      /* verificando codigo ... */
      if(p.getTipo()==Mensagem.codInicio) {
	 System.out.println("Conectado ao servidor...");
	 System.out.println("Passo = " + p.getPasso());
	 System.out.println("Valor = " + p.getValor());
      }     
   }
   
   // thread para esperar mensagens do servidor com lances novos
   public void run() {
      try {
	 while(true) {
	    String entrada = in.readLine();
	    Mensagem p = Mensagem.stringToMensagem(entrada);
	    this.recebeMensagem(p);
	 }	   
      } catch(Exception e) {
	 // essa Exception nunca vai ser enviada...
      }
   }

   // recebendo mensagens...
   public void recebeMensagem(Mensagem p) {
      switch(p.getTipo()) {
	 // msg de erro
	 case Mensagem.codErro         : System.out.println("Você fez uma oferta invalida...");
				         break;
	// Novo Lance...
	 case Mensagem.codNovoLance    : System.out.println("Novo Lance = " + p.getValor());
					 break;
	// Finalizando leilao				 
	 case Mensagem.codFinal        : System.out.println("Leilao encerrado, Lance vencedor = " + p.getValor());
					 System.exit(1);
					 break;
	// Eu venci....
	 case Mensagem.codVencedor     : System.out.println("Voce foi o vencedor do leilao, Muito Obrigado");					 
					 break;
      }
      System.out.print(">");
   }

   // fica pegando valores do teclado, para enviar ao servidor...
   public void roda() {
      BufferedReader entrada=null;
      try {
	 entrada = new BufferedReader( new InputStreamReader(System.in));      
      } catch (Exception e) {
	 System.out.println("Não foi possível abrir a entrada");
	 System.exit(1);
      }
      while(true) {
	 System.out.print(">");
	 try {
	    String novoLance = entrada.readLine();
	    Mensagem p = Mensagem.lanceCliente(Integer.parseInt(novoLance));
	    out.writeBytes(p.toString());
	 } catch(NumberFormatException e) {
	    System.out.println("Digite valores corretos");
	 } catch(Exception e) {
	    System.out.println("Impossivel mandar mensagens ao servidor");
	    System.exit(1);
	 }

      }
   }

   public static void main(String[] args) throws Exception {
      try {
	 BufferedReader entrada = new BufferedReader(new InputStreamReader(System.in));
	 System.out.println("Digite o endereco do Servidor de Leilao" );
	 Cliente p = new Cliente(entrada.readLine(),8080);
	 new Thread(p).start();
         p.roda();
      } 
      catch(Exception e) {
	 System.out.println("Erro conectando-se ao servidor...");
      }
   }

}
